<?php
/**
 * Created by JetBrains PhpStorm.
 * User: alanoliveira
 * Date: 28/03/14
 * Time: 16:12
 * To change this template use File | Settings | File Templates.
 */

namespace Financeiro\Logic\ContasPagar\Cnab\Remessa;


use Exception;
use Financeiro\Entities\Cnabpag;
use Fiscal\Entities\Boletpagar;
use Geral\Entities\Banco;
use Geral\Entities\Empresa;
use Geral\Entities\Municipio;
use Geral\Entities\Filial;
use Geral\Entities\Contcorr;
use Geral\Entities\Tipobaixapag;
use Geral\Util\DateUtil;
use Fiscal\Entities\Contapagar;
use Financeiro\Util\Util;

/**
 * Class GeraCnabItaú
 * @package Financeiro\Logic\ContasPagar\Cnab\Remessa
 */
class GeraCnabItau extends GeraCnab
{
    CONST NUM_LOTE_HEADER_ARQUIVO = '0000';
    CONST ID_TP_ARQUIVO_HEADER_ARQUIVO = '0';
    CONST LAYOUT_ARQUIVO = '080';
    CONST ID_TP_ARQUIVO_HEADER_LOTE = '1';
    CONST ID_TP_ARQUIVO_SEGMENTO = '3';
    CONST ID_TP_ARQUIVO_TRAILER_LOTE = '5';
    CONST TIPO_INS_JUR_CPF = 1;
    CONST TIPO_INS_JUR_CNPJ = 2;
    CONST NOME_BANCO = 'ITAU';
    CONST CDBANCO = '341';
    CONST REMESSA = 1;
    CONST RETORNO = 2;
    CONST VERSAO_LAYOUT = '060';
    CONST TIPO_OPERACAO_CREDITO = 'C';
    CONST TIPO_SERVICO = '20';
    CONST VERSAO_LOTE = '040';
    CONST VERSAO_LOTE_TITULO_BANC = '030';
    CONST SEGMENTO_A = 'A';
    CONST EXCLUSAO = 'E';
    CONST INCLUSAO = 'I';
    CONST BLOQUEIO = 'B';
    CONST MOEDA = 'BRL';
    CONST SEGMETO_B = 'B';
    CONST LOTE_SERVICO_TRAILER_ARQUIVO = '9999';
    CONST ID_TP_ARQUIVO_TRAILER = '9';
    CONST SEGMENTO_J = 'J';

    private $num_header_lote = 0;
    private $seqRegLote = 0;
    private $qtdRegiLote = 0;  //Qtde de registro do lote
    private $qtdRegiArq = 0; //Quantidade de registro do arquivo
    private $qtLote = 0; //Quantidade de Lotes no arquivo
    private $vrPagtLote = 0.00; // Somatório dos valores de inclusao do lote
    private $formapgto;

    /**
     * @param Cnabpag $remessa
     * @throws Exception
     */
    public function __construct(Cnabpag $remessa){

        try {
            $this->nrorg = $remessa->getNrorg();

            $this->remessa = $remessa;

            // Carrega os atributos Filial e Empresa
            $this->filial = Filial::findOneByCdfilialAndNrorg($remessa->getCdfilial(), $remessa->getNrorg());
            $this->empresa = Empresa::findOneByCdempresaAndNrorg($remessa->getCdempresa(), $remessa->getNrorg());

            // Busca dados referente a filial se foi informada
            if ($remessa->getCdfilial()) {
                if ($this->filial->getIdtpijurfili() == Filial::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = $this->filial->getNrinsjurfili();
                $this->nomeempresa = $this->filial->getNmfilial();
            } // Se nao busca dados da Empresa
            else {
                if ($this->empresa->getIdtpijurempr() == Empresa::TIPO_INSCRICAO_FISICA)
                    $this->tipoinscricao = self::TIPO_INS_JUR_CPF;
                else
                    $this->tipoinscricao = self::TIPO_INS_JUR_CNPJ;

                $this->inscricaojuridica = $this->empresa->getNrinsjurempr();
                $this->nomeempresa = $this->empresa->getNmempresa();
            }
            /** @var contcorr Contcorr */
            $this->contcorr = Contcorr::findOneByCdreducontAndNrorg($remessa->getCdreducont(), $remessa->getNrorg());
        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @param $caminhoArquivo
     * @param $boletos
     * @param bool $autorizaAgendamento - Parámetro utilizando quando o banco for Itaú
     * @throws \Exception
     * @internal param $remessa
     * @return mixed
     */

    public function geraArquivo($caminhoArquivo, $boletos, $autorizaAgendamento = false){
        try{
            tk_set_env_var('BOLETOS', $boletos);
            $file = fopen($caminhoArquivo.'.txt', "a+");
            $headerarquivo = self::geraHeaderArquivo();
            fprintf($file, "%s",$headerarquivo);

            // Agrupar boletos por tipo de baixa e por banco favorecido
            $boletosAgrup = self::agrupaBoletos($boletos, $file);

            //Percorre as formas de pagamento
            foreach($boletosAgrup as $segmento => $boletosFormaPgo){
                //Percorre os bancos
                foreach($boletosFormaPgo as $key => $boletosBco)
                {
                    $headerlote  = self::geraHeaderLote($key);
                    fprintf($file, "%s",$headerlote);
                    $somaboletos = 0;
                    foreach($boletosBco as $boleto)
                    {
                        if($segmento == 'SEGMENTOA')
                            $segmento = self::geraSegmentoA($boleto);
                        if($segmento == 'SEGMENTOJ')
                            $segmento = self::geraSegmentoJ($boleto);
                        if($segmento == 'SEGMENTOO')
                            $segmento = self::geraSegmentoO($boleto);
                        fprintf($file, "%s",$segmento);
                        $somaboletos = $boleto->getVrbolpag();
                    }
                    tk_set_env_var('SOMAVALORES', $somaboletos);
                    $trailerLote = self::geraTraillerLote();
                    fprintf($file, "%s",$trailerLote);
                }
            }
            $trailerArquivo = self::geraTrailerArquivo();
            fprintf($file, "%s",$trailerArquivo);
            fclose($file);
        }
        catch (\Exception $e){
            fclose($file);
            throw $e;
        }
    }

    /** Retorna array de Boletos Agrupados por Forma de Pagamento e por código do banco favorecido*/
    private function agrupaBoletos($boletos){
        $boletosAgroup = array();

        $segmentoJ = array(Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);

        $segmentoO = array(Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS);

        /** @var $boleto Boletpagar */
        foreach ($boletos as $boleto)
        {
            $idbaixautilp = $boleto->getTpbaixapag()->getIdbaixautilp();

            if(in_array($idbaixautilp,$segmentoJ))
            {
                $cdbanco = self::getBanco($boleto,Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO);

                if($cdbanco == (GeraCnab::CODIGO_COMPENSACAO_ITAU))
                {
                    $boletosAgroup['SEGMENTOJ'] [GeraCnab::CODIGO_COMPENSACAO_ITAU] [] = $boleto;
                }
                else
                {
                    $boletosAgroup['SEGMENTOJ'] ['OUTROS_BANCOS'] [] = $boleto;
                }

            }
            else
            {
                if(in_array($idbaixautilp,$segmentoO))
                {
                    $cdbanco = self::getBanco($boleto,Tipobaixapag::TIPO_BAIXA_PAG_ORDEM_TRIB_COD_BARRAS);

                    if($cdbanco == (GeraCnab::CODIGO_COMPENSACAO_ITAU))
                    {
                        $boletosAgroup['SEGMENTOO'] [GeraCnab::CODIGO_COMPENSACAO_ITAU] [] = $boleto;
                    }
                    else
                    {
                        $boletosAgroup['SEGMENTOO'] ['OUTROS_BANCOS'] [] = $boleto;
                    }

                }
                else
                {
                    $cdbanco = self::getBanco($boleto);

                    if($cdbanco == (GeraCnab::CODIGO_COMPENSACAO_ITAU))
                    {
                        $boletosAgroup['SEGMENTOA'] [GeraCnab::CODIGO_COMPENSACAO_ITAU] [] = $boleto;
                    }
                    else
                    {
                        $boletosAgroup['SEGMENTOA'] ['OUTROS_BANCOS'] [] = $boleto;
                    }
                }
            }
        }
        return $boletosAgroup;
    }


    // Registro Header de Arquivo
    /**
     * @throws Exception
     * @return string
     */
    public function geraHeaderArquivo()
    {

        try {

            $this->qtdRegiArq = 0;
            $this->qtdRegiArq++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;

            //Número sequêncial do Lote - Posição: 4 a 7 - Picture: 9(4)
            $linha.= self::NUM_LOTE_HEADER_ARQUIVO;

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha.= self::ID_TP_ARQUIVO_HEADER_ARQUIVO;

            //Filler - Posição: 9 a 14 - Picture: X(9)
            $linha.= self::preenche_brancos('',6);

            //Layout de Arquivo - Posição: 15 a 17 - Picture: 9(3)
            $linha.= self::LAYOUT_ARQUIVO;

            //Tipo inscricao da Empresa - Posição: 18 a 18 - Picture: 9(1)
            $linha.= $this->tipoinscricao;

            //CNPJ empresa debitada - Posição: 19 a 32 - Picture: 9(14)
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);

            //Filler - Posição: 33 a 52 - Picture: X(9)
            $linha.= self::preenche_brancos('',20);

            //Número da agência centralizadora - Posição: 53 a 57 - Picture: 9(05)
            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);

            //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
            $linha.= ' ';

            //Número da conta - Posição: 59 a 70 - Picture: 9(12)
            //Obs: A última posição da C/C é reservada para o DAC
            $linha.= str_pad(substr(trim(self::getContaCorrenteSemDigito()),0,12),12,'0',STR_PAD_LEFT);

            //Digito verificador da Agência - Posição: 71 a 71 - Picture: X(1)
            $linha.= ' ';

            //DAC da agência/Conta debitada - Posição: 72 a 72 - Picture: X(1)
            $linha.= substr(self::getDigitoContaCorrente(),0,1);

            //Nome da filial ou Empresa - Posição: 73 a 102 - Picture: X(30)
            $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomeempresa)),0,30),30);

            //Nome do Banco - Posição: 103 a 132 - Picture: X(30)
            $nmbanco = self::NOME_BANCO;
            $linha.= self::preenche_brancos(Util::campoCnabValido($nmbanco),30);


            //Filler - Posição: 133 a 142 - Picture: X(10)
            $linha.= self::preenche_brancos('',10);

            //Código remessa/Retorno (1 = Remessa, 2 = Retorno) - Posição: 143 a 143 - Picture: 9(1)
            $linha.= self::REMESSA;

            //Data da geração do arquivo - Posição: 144 a 151 - Picture: 9(8)
            $linha.= self::getDataGeracao();

            //Hora de geração do arquvio - Posição: 152 a 157 - Picture 9(6)
            $linha.= self::getHorarioAtual();

            //Zeros - Posição: 158 a 166 - Picture: X(9)
            $linha.= str_repeat('0',9);

            //Zeros - Posição: 167 a 171 - Picture: X(5)
            $linha.= str_repeat('0',5);

            //Filler - Posição: 172 a 240 - Picture: X(69)
            $linha.= self::preenche_brancos('',69);

            return $linha."\r\n";
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $cdbanco
     * @param \Fiscal\Entities\Boletpagar $boleto
     * @throws \Exception
     * @internal param $formaLancamento
     * @return string
     */
    public function geraHeaderLote($cdbanco, Boletpagar $boleto = null){

        try{
            $this->vrPagtLote = 0;
            $this->qtdRegiLote = 0;
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;
            $this->qtLote++;

            //Código do banco - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CDBANCO;

            //Número sequêncial do Lote - Posição: 4 a 7 - Picture: 9(4)
            $linha.= str_pad(++$this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha.= self::ID_TP_ARQUIVO_HEADER_LOTE;

            //Tipo de operação (Fixo "C" = Crédito) - Posição: 9 a 9 - Picture: X(01)
            $linha.= self::TIPO_OPERACAO_CREDITO;

            //Tipo de Serviço - Posição: 10 a 11 - Picture: 9(02)
            $formaPagto = self::formaPagamento($cdbanco);
            if (($formaPagto == '16')or ($formaPagto == '17') or ($formaPagto == '18') or
                ($formaPagto == '19') or ($formaPagto == '21') or
                ($formaPagto == '25') or ($formaPagto == '27') or
                ($formaPagto == '35') or ($formaPagto == '91'))
                $linha.= '22';
            else
                $linha.= '20';

            //Forma de lançamento - Posição: 12 a 13 - Picture: 9(02)
            $linha.= $formaPagto;

            //Número da versão no layout do lote - Posição: 14 a 16 - Picture: 9(03)
            if (tk_get_env_var('FORMAPAGAMENTO') == '30' || tk_get_env_var('FORMAPAGAMENTO') == '31')
                $linha.= self::VERSAO_LOTE;
            else
                $linha.= self::VERSAO_LOTE_TITULO_BANC;

            //Filler - Posição: 17 a 17 - Picture: X(01)
            $linha .= ' ';

            //Tipo inscricao da Empresa - Posição: 18 a 18 - Picture: 9(1)
            $linha.= $this->tipoinscricao;

            //CNPJ empresa debitada - Posição: 19 a 32 - Picture: 9(14)
            $linha.= str_pad(trim($this->inscricaojuridica),14,'0',STR_PAD_LEFT);

            //Código do convênio no banco
            //Posição: 33 a 52
            //Picture: X(20)
            $identpag = self::getIdentPag();
            if($identpag){
                $linha.= str_pad(trim($identpag),20);
            }else{
                $linha.= self::preenche_brancos($identpag,20);
            }

            //Número da agência centralizadora - Posição: 53 a 57 - Picture: 9(05)
            $linha.= str_pad(substr(trim($this->contcorr->getCdagencia()),0,-1),5,'0',STR_PAD_LEFT);

            //Digito verificador da Agência - Posição: 58 a 58 - Picture: X(1)
            $linha.= ' ';

            //Número da conta - Posição: 59 a 70 - Picture: 9(12)
            //Obs: A última posição da C/C é reservada para o DAC
            $linha.= str_pad(substr(trim(self::getContaCorrenteSemDigito()),0,12),12,'0',STR_PAD_LEFT);

            //Digito Verificador da Conta - Posição: 71 a 71 - Picture: X(1)
            $linha.= ' ';

            //DAC da agência/Conta debitada - Posição: 72 a 72 - Picture: 9(1)
            $linha.= substr(self::getDigitoContaCorrente(),0,1);

            //Nome da filial ou Empresa - Posição: 73 a 102 - Picture: X(30)
            $linha.= self::preenche_brancos(substr(trim(Util::campoCnabValido($this->nomeempresa)),0,30),30);

            //Campo de mensagem genérica - Posição: 103 a 142 - Picture: X(40)
            $linha.= str_repeat(' ', 40);

            //Endereço da Empresa - Posição: 143 a 172 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido($this->empresa->getDsendeempr()), 30);

            //Campo de mensagem genérica - Posição: 173 a 177 - Picture: X(5)
            $linha.= str_repeat('0', 5);

            //Campo de mensagem genérica - Posição: 178 a 192 - Picture: X(15)
            $linha.= str_repeat(' ', 15);

            //Cidade da Empresa - Posição: 193 a 212 - Picture: X(20)
            $municipio = Municipio::findOneBySgestadoAndCdpaisAndCdmunicipio($this->empresa->getSgestado(), $this->empresa->getCdpais(), $this->empresa->getCdmunicipio());
            $linha.= str_pad($municipio->getNmmunicipio(), 20);

            //Cep da Empresa - Posição: 213 a 220 - Picture: X(8)
            $linha.= Util::campoCepTel($this->empresa->getNrcepempr());

            //Estado da Empresa - Posição: 221 a 222 - Picture: X(8)
            $linha.= $this->empresa->getSgestado();

            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',138);

            return $linha."\r\n";


        }catch(\Exception $e){
            throw $e;
        }
    }

    public function formaPagamento($cdbanco)
    {
        $boletos = tk_get_env_var('BOLETOS');
        /** @var $boleto Boletpagar */
        foreach ($boletos as $boleto){
            $idbaixautilp = $boleto->getTpbaixapag()->getIdbaixautilp();
        }

        switch ($idbaixautilp)
        {
            case 'B':
                if($cdbanco == self::CODIGO_COMPENSACAO_ITAU)
                    $this->formapgto = '30';
                else
                    $this->formapgto = '31';
                break;
            case 'G': $this->formapgto = '17';
                break;
            case 'H': $this->formapgto = '18';
                break;
            case 'I': $this->formapgto = '21';
                break;
            case 'J': $this->formapgto = '21';
                break;
            case 'K': $this->formapgto = '27';
                break;
            case 'N': $this->formapgto = '19';
                break;
            case 'O': $this->formapgto = '10';
                break;
            case 'Q': $this->formapgto = '91';
                break;
            case 'R': $this->formapgto = '13';
                break;
            case 'U': $this->formapgto = '35';
                break;
            case 'V': $this->formapgto = '25';
                break;
            case 'X': $this->formapgto = '22';
                break;
            default:
                if ($boleto->getCdbancbolp() == '341')
                {
                    $this->formapgto = '01';

                }
                else if ($boleto->getVrfinalbpag <= $boleto->getVrmaxdoccnab)
                    $this->formapgto = '03';
                else
                    $this->formapgto = '41';
                break;
        }
        tk_set_env_var('FORMAPAGAMENTO', $this->formapgto);
        return $this->formapgto;
    }

    /**
     * retorna a data atual no formato ddmmYYYY
     * @return string
     */
    private function getDataGeracao(){
        $dataatual = DateUtil::getDataAtual();
        return $dataatual->format('d').$dataatual->format('m').$dataatual->format('Y');
    }

    /**
     * @return string
     */
    private function getHorarioAtual(){
        $dataatual = DateUtil::getDataAtual();
        return $dataatual->format('H').$dataatual->format('i').$dataatual->format('s');
    }

    /** Retorna Conta Corrente sem a ultima posição (dígito)
     * @return string
     */
    private function getContaCorrenteSemDigito(){
        $contacorrente = $this->contcorr->getCdcontcorr();
        return substr($contacorrente,0,-1);
    }

    /**
     * Retorna a ultima posição da Conta Corrente (Dígito)
     * @return int|string
     */
    private function getDigitoContaCorrente(){
        $contacorrente = $this->contcorr->getCdcontcorr();
        return substr($contacorrente,-1);
    }

    /**
     * @param $valor - valor a ser alterado
     * @param $tamanho - tamanho total da string desejado
     * @return string
     */
    private function preenche_brancos($valor, $tamanho){
        return str_pad($valor, $tamanho);
    }


     /**
     * @param $nrlanbolpag
     * @param $cdfilibpag
     * @param $nrorg
     * @return Contapagar
     */
    private function getContaPagar($nrlanbolpag, $cdfilibpag, $nrorg){
        /** @var $contaPagar Contapagar */
        $contaPagar = Contapagar::findOneByNrlanbolpagAndCdfilibpagAndNrorg($nrlanbolpag, $cdfilibpag, $nrorg);
        return $contaPagar;
    }


    /**
     * @internal param $cdbanco
     * @throws Exception
     * @return string
     * @internal param $caminho
     * @internal param $nrorg
     */
    public function geraTraillerLote(){

        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do banco na compensação - Posição: 1 a 3 - Picture: 9(3)
            $linha = self::CODIGO_COMPENSACAO_ITAU;

            //Número de sequência do lote - Posição: 4 a 7 - Picture: 9(4)
            $linha .= str_pad($this->num_header_lote, 4, '0', STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha .= self::ID_TP_ARQUIVO_TRAILER_LOTE;

            //Filler - Posição: 9 a 17 - Picture: X(9)
            $linha .= str_repeat(' ', 9);

            //Quantidade de Registro do lote - Posição: 18 a 23 - Picture: 9(6)
            $linha .= str_pad($this->qtdRegiLote, 6, '0', STR_PAD_LEFT);

            //Somatorio dos valores a serem lançados - Posição: 24 a 41 - Picture: 9(16)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format(tk_get_env_var('SOMAVALORES'),2)
                                         ),18,'0',STR_PAD_LEFT);

            //Somatório da quantidade de moeda - Posição: 42 a 56 - Picture: 9(18)
            $linha .= str_repeat('0', 18);

            //Filler - Posição: 57 a 230 - Picture: X(165)
            //Ocorrências retorno - Posição: 231 a 240 - Picture: X(10)
            $linha .= str_repeat(' ', 174);

            //Ocorrências - Posição: 231 a 240 - Picture: X(10)
            $linha .= str_repeat(' ', 10);

            //Zera o contador de registros do lote
            $this->qtdRegiLote = 0;


            $this->seqRegLote = 0;

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }

    }

    /**
     * @return string
     * @throws \Exception
     */
    public function geraTrailerArquivo(){

        try { //Quantidade de registro do arquivo
            $this->qtdRegiArq++;

            //Código do banco na compensação - Posição: 1 a 3 - Picture: 9(3)
            $linha = self::CODIGO_COMPENSACAO_ITAU;

            //Número de sequência do lote - Posição: 4 a 7 - Picture: 9(4)
            //Tipo de registro - Posição: 8 a 8 - Picture: 9(1)
            $linha .= self::LOTE_SERVICO_TRAILER_ARQUIVO . self::ID_TP_ARQUIVO_TRAILER;

            //Filler - Posição: 9 a 17 - Picture: X(9)
            $linha .= str_repeat(' ', 9);

            //Quantidade de lotes do arquivo - Posição: 18 a 23 - Picture: 9(6)
            $linha .= str_pad($this->qtLote, 6, '0', STR_PAD_LEFT);

            //Qtde total de registros do arquivo - Posição: 24 a 29 - Picture: 9(6)
            $linha .= str_pad($this->qtdRegiArq, 6, '0', STR_PAD_LEFT);

            //Filler - Posição: 30 a 240 - Picture: X(211)
            $linha .= str_repeat(' ', 211);

            return $linha . "\r\n";
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function geraSegmentoJ($boleto)
    {
        try{
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_ITAU;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            //Código do segmento reg. detalhe - Posição: 14 a 14 - Picture: X(01)
            $linha.= 'J';

            //Tipo de movimento - Posição: 15 a 17 - Picture: 9(3)
            if (tk_get_field_value('IDOPECNABPAG') == 'E')
                $linha.= '999';
            else
                $linha.= '000';

            //Código de Barras - Posição: 18 a 61 - Picture: X(44)
            if($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
                if($boleto->getCdbarrapag()){
                    $cdbanco = str_pad(substr($boleto->getCdbarrapag(),0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($boleto->getCdbarrapag(),3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($boleto->getCdbarrapag(),4,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($boleto->getCdbarrapag(),5,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($boleto->getCdbarrapag(),19,25),25,'0',STR_PAD_LEFT);
                } else {
                    $linhaBarra = '';
                    $dslinhbarpag = $boleto->getDslinhbarpag();
                    for($i = 0;$i<strlen($dslinhbarpag);$i++){
                        if(in_array($dslinhbarpag[$i],array('0','1','2','3','4','5','6','7','8','9'))){
                            $linhaBarra.= $dslinhbarpag[$i];
                        }
                    }
                    $cdbanco = str_pad(substr($linhaBarra,0,3),3,'0',STR_PAD_LEFT);
                    $cdmoeda = str_pad(substr($linhaBarra,3,1),1,'0',STR_PAD_LEFT);
                    $cddigito = str_pad(substr($linhaBarra,32,1),1,'0',STR_PAD_LEFT);
                    $valor = str_pad(substr($linhaBarra,33,14),14,'0',STR_PAD_LEFT);
                    $cdcampo = str_pad(substr($linhaBarra,4,25),25,'0',STR_PAD_LEFT);
                }
            }
            //Código do Banco(Cd. Barra) - Posição: 18 a 20 - Picture: 9(3)
            $linha.= $cdbanco;

            //Código da Moeda(Cd. Barra) - Posição: 21 a 21 - Picture: 9(1)
            $linha.= $cdmoeda;

            //Dígito verificador(Cd. Barra) - Posição: 22 a 22 - Picture: 9(1)
            $linha.= $cddigito;

            //Valor (Cd. Barra) - Posição: 23 a 36 - Picture: 9(14)
            $linha.= $valor;

            //Campo Livre (Cd. Barra) - Posição: 37 a 61 - Picture: 9(25)
            $linha.= $cdcampo;

            //Nome do cedente - Posição: 62 a 91 - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);

            //Data do vencimento - Posição: 92 a 99 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do Título - Posição: 100 a 114 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Valor do Desconto + Abatimento - Posição: 115 a 129 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0',15);

            //Valor do Acrescimo - Posição: 130 a 144 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0',15);

            //Data do pagamento - Posição: 145 a 152 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Valor do pagto - Posição: 153 a 167 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Quantidade de moeda - Posição:  168 a 182 - Picture: 9(15)
            $linha.= str_repeat('0',15);

            //Cód. Refer. atribuído pelo sacado - Posição: 183 a 202 - Picture: X(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),
                $boleto->getCdfilial(),
                $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);

            //Complemento - Posição: 203 a 215 - Picture: X(13)
            $linha.= str_repeat(' ',13);

            //Nosso Número - Posição: 216 a 230 - Picture: X(15)
            $linha.= str_repeat(' ',15);

            //Ocorrências - Posição: 231 a 240 - Picture: X(10)
            $linha.= str_repeat(' ',10);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }

    }

    public function geraSegmentoO($boleto)
    {
        try{
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_ITAU;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            $linha.= 'O';
            //Tipo de movimento - Posição: 15 a 17 - Picture: 9(3)
            if (tk_get_field_value('IDOPECNABPAG') == 'E')
                $linha.= '999';
            else
                $linha.= '000';

            //Código de Barras - Posição: 18 a 65 - Picture: X(44)
            if($boleto->getTpbaixapag()->getIdbaixautilp() == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
                if($boleto->getCdbarrapag()){
                    $cdbanco = $boleto->getCdbarrapag();
                } else {
                    $linhaBarra = '';
                    $dslinhbarpag = $boleto->getDslinhbarpag();
                    for($i = 0;$i<strlen($dslinhbarpag);$i++){
                        if(in_array($dslinhbarpag[$i],array('0','1','2','3','4','5','6','7','8','9'))){
                            $linhaBarra.= $dslinhbarpag[$i];
                        }
                    }
                    $cdbanco = $boleto->getCdbarrapag();
                }
            }

            //Nome do favorecido - Posição: 66 a 95 - Picture: X(30)
            $linha.= Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30));

            //Data do vencimento - Posição: 96 a 103 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Campo: Tipo Moeda - Posição: 104 a 106 - Picture: X(03)
            $linha.= 'REA';

            //Quantidade Moeda - Posição: 107 a 121 - Picture: X(03)
            //--------------------------------------

            //Valor do pagto - Posição: 122 a 136 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);

            //Data do pagamento - Posição: 137 a 144 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');

            //Zeros - Posição: 160 a 162 - Picture: X(3)
            $linha.= str_repeat(' ', 3);

            //Nota fiscal - Posição: 163 a 171 - Picture: X(9)

            //Zeros - Posição: 172 a 174 - Picture: X(3)
            $linha.= str_repeat(' ', 3);

            //Cód. Refer. atribuído pelo sacado - Posição: 175 a 194 - Picture: X(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),
                $boleto->getCdfilial(),
                $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);

            //Zeros - Posição: 195 a 215 - Picture: X(21)
            $linha.= str_repeat(' ', 21);

            //Nosso Número - Posição: 216. a 230 - Picture: X(21)
            $linha.= str_repeat(' ', 21);

            //Ocorrências - Posição: 231 a 240 - Picture: X(21)
            $linha.= str_repeat(' ', 21);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraSegmentoA($boleto)
    {
        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_ITAU;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            $linha.= 'A';
            //Tipo de movimento - Posição: 15 a 17 - Picture: 9(3)
            if (tk_get_field_value('IDOPECNABPAG') == 'E')
                $linha.= '999';
            else
                $linha.= '000';
            //Complemento de registro - Posição: 18 a 20 - Picture: 9(3)
            $linha.= '000';
            //Código do Banco - Posição: 21 a 23 - Picture: 9(3)
            $linha.= self::CODIGO_COMPENSACAO_ITAU;
            //Código da Agência conta Favorecido - Posição:
            $linha.= '0'.substr($this->contcorr->getCdagencia(), -4).' '.str_repeat('0', 6);
            $linha.= str_pad(substr($this->contcorr->getCdcontcorr(), 0, 5), 6, 0, STR_PAD_LEFT).' '.self::getDigitoContaCorrente();
            //Nome do cedente - Posição:  a  - Picture: X(30)
            $linha.= str_pad(Util::campoCnabValido(substr($boleto->getForneced()->getNmrazsocforn(),0,30)),30);
            //Seu número - Posição: 74 a 93 - Picture: X(20)
            $contaPagar = self::getContaPagar($boleto->getNrlanbolpag(),
                $boleto->getCdfilial(),
                $boleto->getNrorg());
            $linha.= str_pad(trim($contaPagar->getCdfilibpag().$boleto->getNrlanbolpag()),20,' ',STR_PAD_RIGHT);
            //Data do pagamento - Posição: 094 a 101 - Picture: 9(8)
            $linha.= $boleto->getDtvenbolpag()->format('dmY');
            //Campo: Tipo Moeda - Posição: 102 a 104 - Picture: X(03)
            $linha.= 'REA';
            //Zeros - Posição: 105 a 119 - Picture: X(15)
            $linha.= str_repeat('0', 15);
            //Valor do pagto - Posição: 120 a 134 - Picture: 9(13)V9(2)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag(),2)
            ),15,'0',STR_PAD_LEFT);
            //Campo: Número atribuído pelo banco - Posição: 135 a 149 - Picture: X(15) - Conteúdo: Brancos
            $linha.= str_repeat(' ', 15);
            //Brancos: Posição: 150 a 154 - Picture: X(5) - Conteúdo: Brancos
            $linha.= str_repeat(' ', 5);

            //Data Real Efetivação - Posição: 155 a 162 - Picture: 9(8)
            $linha.= str_repeat('0', 8);

            //Valor Real Efetivação - Posição: 163 a 177 - Picture: 9(13)V9(2)
            $linha.= str_repeat('0', 15);

            //Finalidade detalhe - Posição: 178 a 195 - Picture: X(18)
            $linha.= str_repeat(' ', 18);

            //Brancos - Posição: 196 a 197 - Picture: X(2)
            $linha.= str_repeat(' ', 2);

            //Numero de DOC/OP/CHEQUE - Posição: 198 a 203 - Picture: 9(6)
            $linha.= str_repeat('0', 6);

            //Numero de Inscriçao - Posição: 204 a 217 - Picture: 9(14)
            if($boleto->getNrinscnabbop()){
                $linha.= str_pad(substr($boleto->getForneced()->getNrinsjurforn(),0,14),14,'0');
            }

            //Brancos - Posição: 218 a 229 - Picture: X(12)
            $linha.= str_repeat(' ', 12);

            //Aviso - Posição: 230 a 230 - Picture: X(1)
            $linha.= '0';

            //Ocorrências - Posição: 231 a 240 - Picture: X(21)
            $linha.= str_repeat(' ', 21);

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function geraSegmentoN($boleto)
    {
        try {
            $this->qtdRegiLote++;
            $this->qtdRegiArq++;

            //Código do Banco na Compensação - Posição: 1 a 3 - Picture: 9(03)
            $linha = self::CODIGO_COMPENSACAO_ITAU;

            ///Numero do lote de serviço ao qual pertence - Posição: 4 a 7 - Picture: 9(04)
            $linha.= str_pad($this->num_header_lote,4,'0',STR_PAD_LEFT);

            //Tipo de registro - Posição: 8 a 8 - Picture: 9(01)
            $linha.= self::ID_TP_ARQUIVO_SEGMENTO;

            //Nº de sequencial do registro dentro do lote - Posição: 9 a 13 - Picture: 9(05)
            $linha.= str_pad(++$this->seqRegLote,5,'0',STR_PAD_LEFT);

            $linha.= 'N';
            //Tipo de movimento - Posição: 15 a 17 - Picture: 9(3)
            if (tk_get_field_value('IDOPECNABPAG') == 'E')
                $linha.= '999';
            else
                $linha.= '000';

            //Identificação do tributo - Posição: 18 a 19 - Picture: 9(02)
            if ($formaPagto = '16')
                self::darfNormal($linha, $boleto);
            else
                if($formaPagto = '17')
                    self::GPS($linha, $boleto);
                elseif ($formaPagto = '18')
                    self::darfSimples($linha, $boleto);


            //Código da receita - Posição: 20 a 23 - Picture: 9(04)
            $cdreceitaguia = $boleto->getCdreceitaguia();
            if (!Empty($cdreceitaguia))
                $linha.= str_pad(trim($cdreceitaguia), 4, '0', STR_PAD_LEFT );
            else
                $linha.= str_repeat('0', 4);

            //Tipo inscrição empresa debitada - Posição: 24 a 24 - Picture: 9(1)
            $empresa = empresa::findOneByNrorgAndCdempresa(tk_get_field_value('NRORG'), tk_get_field_value('CDEMPRESA'));
            if ($empresa->getIdtpijurempr() == 'F')
                $linha.= '1'; //CPF
            else
                $linha.= '2'; //CNPJ

           //CPF ou CNPJ do contribuinte  - Posição: 25 a 38 - Picture: 9(14)
           $linha.= str_pad(Copy(trim(is_numeric($empresa->getNrinsjurempr())), 1, 14), 14);

           //Período de Apuração - Posição: 39 a 46 - Picture: 9(08)
           if ($boleto->getDtapuracguia() != 0)
               $linha.= self::formataData($boleto->getDtapuracguia());
           else
              $linha.= str_repeat('0', 8);

           //Número de referência - Posição: 47 a 63 - Picture: 9(17)
            $nrrefeguia = $boleto->getNrrefeguia();
           if (!Empty($nrrefeguia))
               $linha.= str_pad(trim(is_numeric($nrrefeguia)), 17, '0', STR_PAD_LEFT);
           else
               $linha.= str_repeat('0', 17);

           //Valor Principal - Posição: 64 a 77 - Picture: 9(12)V2(02)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag())), 14);

           //Valor da Multa - Posição: 78 a 91 - Picture: 9(12)V2(02)
            $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag())), 14);

           //Valor dos Juros/ Encargos - Posição: 92 a 105 - Picture: 9(12)V2(02)
           $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag())), 14);

           //Valor total a ser pago - Posição: 106 a 119 - Picture: 9(12)V2(02)
           $linha.= str_pad(str_replace(array('.',','),'', number_format($boleto->getVrfinalbpag())), 14);

           //Data de Vencimento - Posição: 120 a 127 - Picture: 9(08)
           $linha.= self::formataData($boleto->getDtapuracguia());

           //Data do Pagamento - Posição: 128 a 135 - Picture: 9(08)
           $linha.= self::formataData($boleto->getDtapuracguia());

           //Brancos - Posição: 136 a 165 - Picture: X(30)
           $linha.= str_repeat(' ', 30);

           //Nome do Contribuinte - Posição: 166 a 195 - Picture: 9(30)
           $linha.= str_pad(Util::campoCnabValido(Copy($empresa->getNmrazsocempr(), 1, 30)), 30, '0');

            return $linha . "\r\n";

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function darfNormal($linha, $boleto)
    {
        //Identificação do tributo - Posição: 18 a 19 - Picture: 9(02)
        $linha.= '02'; //DARF  Normal

        //Código da receita - Posição: 20 a 23 - Picture: 9(04)
        $cdreceitaguia = $boleto->getCdreceitaguia();
        if (!Empty($cdreceitaguia))
            $linha.= str_pad(trim(is_numeric($cdreceitaguia)), 4, '0', STR_PAD_LEFT);
        else
            $linha.= str_repeat('0', 4);

        //Tipo inscrição empresa debitada - Posição: 24 a 24 - Picture: 9(1)
        $empresa = empresa::findOneByNrorgAndCdempresa(tk_get_field_value('NRORG'), tk_get_field_value('CDEMPRESA'));
        if ($empresa->getIdtpijurempr() == 'F')
            $linha.= '1'; //CPF
        else
            $linha.= '2'; //CNPJ

        //CPF ou CNPJ do contribuinte  - Posição: 25 a 38 - Picture: 9(14)
        $linha.= str_pad(Copy(trim(is_numeric($empresa->getNrinsjurempr())), 1, 14), 14, '0', STR_PAD_LEFT);

        //Período de Apuração - Posição: 39 a 46 - Picture: 9(08)
        if ($boleto->getDtapuracguia() != 0)
            $linha.= self::formataData($boleto->getDtapuracguia());
        else
            $linha.= atr_repeat('0', 8);

        //Número de referência - Posição: 47 a 63 - Picture: 9(17)
        $nrrefeguia = $boleto->getNrrefeguia();
        if (!Empty($nrrefeguia))
            $linha.= str_pad(trim(is_numeric($nrrefeguia), 17), 17, '0', STR_PAD_LEFT);
        else
            $linha.= str_repeat('0', 17);

        //Valor Principal - Posição: 64 a 77 - Picture: 9(12)V2(02)
        $linha.= str_pad(Util::campoCnabValido(number_format('#0.00',$boleto->getVrfinalbpag()), 14),14, STR_PAD_LEFT);

        //Valor da Multa - Posição: 78 a 91 - Picture: 9(12)V2(02)
        $linha.= str_pad(Util::campoCnabValido(number_format('#0.00', $boleto->getVrmultabpag()), 14), 14, STR_PAD_LEFT);

        //Valor dos Juros/ Encargos - Posição: 92 a 105 - Picture: 9(12)V2(02)
        $linha.= str_pad(Util::campoCnabValido(number_format('#0.00', $boleto->getVrmordiabpag()), 14), 14, STR_PAD_LEFT);

        //Valor total a ser pago - Posição: 106 a 119 - Picture: 9(12)V2(02)
        $linha.= str_pad(Util::campoCnabValido(number_format('#0.00', $boleto->getVrfinalbpag()), 14),14, STR_PAD_LEFT);

        //Data de Vencimento - Posição: 120 a 127 - Picture: 9(08)
        $linha.= self::formataData($boleto->getDtvenbolpag());

        //Data do Pagamento - Posição: 128 a 135 - Picture: 9(08)
        $linha.= self::formataData($boleto->getDtvenbolpag());

        //Brancos - Posição: 136 a 165 - Picture: X(30)
        $linha.= str_repeat(' ', 30);

        //Nome do Contribuinte - Posição: 166 a 195 - Picture: 9(30)
        $linha.= str_pad(Util::campoCnabValido(Copy($boleto->getNmrazsocempr(), 1, 30)), 30);

        return $linha;
    }

    public function darfSimples($linha, $boleto)
    {
        //Identificação do tributo - Posição: 18 a 19 - Picture: 9(02)
        $linha.= '03'; //DARF Simples
    }

    public function GPS($linha, $boleto)
    {
        //Identificação do tributo - Posição: 18 a 19 - Picture: 9(02)
        $linha.= '01'; //GPS
    }

    public static function formataData($data){
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

}